# Extending `RagConnect`

To add a new communication protocol, the following locations have to be changed (replace `ABC` and `abc` with the name of the protocol):

Within `ragconnect.base/src/main/resources`:
- Add a new handler `ABCHandler`, if appropriate, similar to the existing handlers
  - If further methods are needed for handler initialization, add a new template `abc.mustache` containing those procedures. Add `{{#usesABC}}{{> abc}}{{/usesABC}}` at the top of `ragconnect.mustache` to use this template
- In `receiveDefinition.mustache` and `sendDefinition.mustache`: add a new case in the switch statement defining the logic to happen for both definitions. If the new protocol is close to a PUSH semantic, follow `mqtt`. If it is closer to PULL semantic, follow `rest`.

Within `ragconnect.base/src/main/jastadd`:
- In `backend/Configuration`:
  - Add a new static boolean flag `usesABC` to indicate whether the protocol is used
- In `backend/Generation`:
  - Add new attributes for type `MRagConnect` for handler attribute and handler field, if needed
  - Add attributes for newly introduced references in changed mustache templates, if any
  - Add a newly constructed handler within the definition of `RagConnect.toMustache` with the needed fields (class name, construction snippet, handler attribute, handler field, the boolean flag you just added to Configuration)
- In `backend/MustacheNodesToYAML`:
  - Add key-value-pair for `usesABC` (and handler, if any)
  - Add key-value-pairs for newly introduced referemces in changed mustache templates, if any

In `ragconnect.base/src/main/java/org/jastadd/ragconnect/compiler/Compiler.java`:
- Add a new choice for `--protocols` similar to the existing ones
- Set the flag `usesABC` if the choice is given.
- Add code to add the handler to the list `handlers` if the choice is given, i.e., if `ASTNode.usesABC`

Furthermore, new test cases are appreciated. They can be added in the [ragconnect.rests repository](https://git-st.inf.tu-dresden.de/jastadd/ragconnect-tests)
